@@ -284,6 +284,24 @@ mx_long_press_action_get_type(void) {
     }
   return enum_type_id;
 }
+/* enumerations from "../mx/mx-kinetic-scroll-view.h" */
+#include "../mx/mx-kinetic-scroll-view.h"
+GType
+mx_kinetic_scroll_view_state_get_type(void) {
+  static GType enum_type_id = 0;
+  if (G_UNLIKELY (!enum_type_id))
+    {
+      static const GEnumValue values[] = {
+        { MX_KINETIC_SCROLL_VIEW_STATE_IDLE, "MX_KINETIC_SCROLL_VIEW_STATE_IDLE", "idle" },
+        { MX_KINETIC_SCROLL_VIEW_STATE_PANNING, "MX_KINETIC_SCROLL_VIEW_STATE_PANNING", "panning" },
+        { MX_KINETIC_SCROLL_VIEW_STATE_SCROLLING, "MX_KINETIC_SCROLL_VIEW_STATE_SCROLLING", "scrolling" },
+        { MX_KINETIC_SCROLL_VIEW_STATE_CLAMPING, "MX_KINETIC_SCROLL_VIEW_STATE_CLAMPING", "clamping" },
+        { 0, NULL, NULL }
+      };
+      enum_type_id = g_enum_register_static (g_intern_static_string ("MxKineticScrollViewState"), values);
+    }
+  return enum_type_id;
+}
 
 /* Generated data ends here */
 
