@@ -36,6 +36,8 @@
 #include "mx-marshal.h"
 #include "mx-texture-cache.h"
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
+
 G_DEFINE_TYPE (MxImage, mx_image, MX_TYPE_WIDGET)
 
 #define MX_IMAGE_GET_PRIVATE(obj)    \
@@ -196,7 +198,7 @@ mx_image_async_data_new (MxImage *parent)
 }
 
 static void
-get_center_coords (CoglObject *tex,
+get_center_coords (CoglHandle  tex,
                    float       rotation,
                    float       aw,
                    float       ah,
@@ -214,11 +216,11 @@ get_center_coords (CoglObject *tex,
 }
 
 static gfloat
-calculate_scale (CoglObject       *texture,
-                 float             rotation,
-                 float             aw,
-                 float             ah,
-                 MxImageScaleMode  mode)
+calculate_scale (CoglHandle       texture,
+                 float            rotation,
+                 float            aw,
+                 float            ah,
+                 MxImageScaleMode mode)
 {
   float bw, bh, tmp, factor;
 
@@ -268,10 +270,9 @@ mx_image_paint (ClutterActor *actor)
   float tex_coords[8];
   MxPadding padding;
   CoglMatrix matrix;
-  gfloat scale = 1, previous_scale = 1;
+  gfloat scale = 1;
   gfloat ratio;
   CoglColor color;
-  gfloat progress;
 
   /* chain up to draw the background */
   CLUTTER_ACTOR_CLASS (mx_image_parent_class)->paint (actor);
@@ -295,25 +296,36 @@ mx_image_paint (ClutterActor *actor)
   ratio = bw/bh;
 
   alpha = clutter_actor_get_paint_opacity (actor);
-
-  /* Paint opacity is applied using a constant on the third layer of the
-   * material. This ensures the paint opacity is applied to both textures. */
   cogl_color_init_from_4ub (&color, alpha, alpha, alpha, alpha);
-  cogl_material_set_layer_combine_constant (priv->material, 2, &color);
+
+  if (priv->old_texture)
+    {
+      /* Paint opacity is applied using a constant on the third layer of the
+       * material. This ensures the paint opacity is applied to both textures. */
+      cogl_material_set_layer_combine_constant (priv->material, 2, &color);
+    }
+  else
+    {
+      cogl_material_set_color (priv->material, &color);
+      cogl_material_set_layer (priv->material, 0, priv->texture);
+    }
 
   /* calculate texture co-ordinates */
   get_center_coords (priv->texture, priv->rotation, aw, ah, tex_coords);
-  get_center_coords (priv->old_texture, priv->old_rotation, aw, ah,
-                     tex_coords + 4);
 
   /* current texture */
   scale = calculate_scale (priv->texture, priv->rotation, aw, ah, priv->mode);
-  previous_scale = calculate_scale (priv->texture, priv->rotation, aw, ah,
-                                    priv->previous_mode);
 
-  progress = clutter_alpha_get_alpha (priv->redraw_alpha);
   if (clutter_timeline_is_playing (priv->redraw_timeline))
-    scale = scale + (previous_scale - scale) * (1 - progress);
+    {
+      gfloat progress, previous_scale;
+
+      previous_scale = calculate_scale (priv->texture, priv->rotation, aw, ah,
+                                        priv->previous_mode);
+
+      progress = clutter_alpha_get_alpha (priv->redraw_alpha);
+      scale = scale + (previous_scale - scale) * (1 - progress);
+    }
 
   cogl_matrix_init_identity (&matrix);
   cogl_matrix_translate (&matrix, 0.5, 0.5, 0);
@@ -329,27 +341,33 @@ mx_image_paint (ClutterActor *actor)
 
 
   /* old texture */
-  scale = calculate_scale (priv->old_texture, priv->old_rotation, aw, ah,
-                           priv->old_mode);
+  if (priv->old_texture)
+    {
+      get_center_coords (priv->old_texture, priv->old_rotation, aw, ah,
+                         tex_coords + 4);
 
-  bw = cogl_texture_get_width (priv->old_texture);
-  bh = cogl_texture_get_height (priv->old_texture);
-  ratio = bw/bh;
+      scale = calculate_scale (priv->old_texture, priv->old_rotation, aw, ah,
+                               priv->old_mode);
 
-  cogl_matrix_init_identity (&matrix);
-  cogl_matrix_translate (&matrix, 0.5, 0.5, 0);
+      bw = cogl_texture_get_width (priv->old_texture);
+      bh = cogl_texture_get_height (priv->old_texture);
+      ratio = bw/bh;
 
-  cogl_matrix_scale (&matrix, 1, ratio, 1);
-  cogl_matrix_rotate (&matrix, priv->old_rotation, 0, 0, -1);
-  cogl_matrix_scale (&matrix, 1, 1 / ratio, 1);
+      cogl_matrix_init_identity (&matrix);
+      cogl_matrix_translate (&matrix, 0.5, 0.5, 0);
 
-  cogl_matrix_scale (&matrix, scale, scale, 1);
+      cogl_matrix_scale (&matrix, 1, ratio, 1);
+      cogl_matrix_rotate (&matrix, priv->old_rotation, 0, 0, -1);
+      cogl_matrix_scale (&matrix, 1, 1 / ratio, 1);
 
-  cogl_matrix_translate (&matrix, -0.5, -0.5, 0);
-  cogl_material_set_layer_matrix (priv->material, 1, &matrix);
+      cogl_matrix_scale (&matrix, scale, scale, 1);
+
+      cogl_matrix_translate (&matrix, -0.5, -0.5, 0);
+      cogl_material_set_layer_matrix (priv->material, 1, &matrix);
+
+    }
 
   cogl_set_source (priv->material);
-
   cogl_rectangle_with_multitexture_coords (padding.left, padding.top,
                                            padding.left + aw, padding.top + ah,
                                            tex_coords,
@@ -679,19 +697,25 @@ mx_image_class_init (MxImageClass *klass)
   mx_image_cache_quark = g_quark_from_static_string ("mx-image-cache");
 }
 
-
 static void
-new_frame_cb (ClutterTimeline *timeline,
-              guint            elapsed_msecs,
-              MxImage         *image)
+create_new_material (MxImage *image,
+                     gdouble  progress)
 {
   MxImagePrivate *priv = image->priv;
   CoglHandle copy;
-  gdouble progress;
   CoglColor constant;
 
-  if (priv->material == COGL_INVALID_HANDLE)
-    return;
+  if (!priv->old_texture)
+    {
+      if (priv->material)
+        cogl_object_unref (priv->material);
+
+      priv->material = cogl_material_new ();
+      cogl_material_set_layer_wrap_mode (priv->material, 0,
+                                         COGL_MATERIAL_WRAP_MODE_CLAMP_TO_EDGE);
+
+      return;
+    }
 
   /* You should assume that a material can only be modified once, after
    * its creation; if you need to modify it later you should use a copy
@@ -699,8 +723,6 @@ new_frame_cb (ClutterTimeline *timeline,
    */
   copy = cogl_material_copy (priv->material);
 
-  progress = clutter_timeline_get_progress (priv->timeline);
-
   /* Create the constant color to be used when combining the two
    * material layers; we use a black color with an alpha component
    * depending on the current progress of the timeline
@@ -716,15 +738,36 @@ new_frame_cb (ClutterTimeline *timeline,
     cogl_object_unref (priv->material);
 
   priv->material = copy;
+}
+
+static void
+new_frame_cb (ClutterTimeline *timeline,
+              guint            elapsed_msecs,
+              MxImage         *image)
+{
+  MxImagePrivate *priv = image->priv;
+  gdouble progress;
+
+  if (priv->material == COGL_INVALID_HANDLE)
+    return;
+
+  progress = clutter_timeline_get_progress (priv->timeline);
+
+  create_new_material (image, progress);
 
   clutter_actor_queue_redraw (CLUTTER_ACTOR (image));
 }
 
 static void
-redraw_timeline_complete (ClutterTimeline *timeline,
-                          MxImage         *image)
+timeline_complete (ClutterTimeline *timeline,
+                   MxImage         *image)
 {
-  image->priv->old_mode = image->priv->mode;
+  if (image->priv->old_texture)
+    {
+      cogl_object_unref (image->priv->old_texture);
+      image->priv->old_texture = NULL;
+    }
+  create_new_material (image, 1.0);
 }
 
 static void
@@ -743,11 +786,11 @@ mx_image_init (MxImage *self)
 
   g_signal_connect (priv->timeline, "new-frame", G_CALLBACK (new_frame_cb),
                     self);
+  g_signal_connect (priv->timeline, "completed", G_CALLBACK (timeline_complete),
+                    self);
 
   g_signal_connect_swapped (priv->redraw_timeline, "new-frame",
                             G_CALLBACK (clutter_actor_queue_redraw), self);
-  g_signal_connect (priv->redraw_timeline, "completed",
-                    G_CALLBACK (redraw_timeline_complete), self);
 
   priv->blank_texture = cogl_texture_new_from_data (1, 1, COGL_TEXTURE_NO_ATLAS,
                                                     COGL_PIXEL_FORMAT_RGBA_8888,
@@ -901,9 +944,14 @@ mx_image_prepare_texture (MxImage  *image)
   cogl_material_set_layer (priv->material, 1, priv->old_texture);
   cogl_material_set_layer (priv->material, 0, priv->texture);
 
-  /* start the cross fade animation */
+  /* start the cross fade animation. When not having a transition duration,
+   * we directly jump forward a create the material corresponding to the end
+   * of the transition animation */
   clutter_timeline_stop (priv->timeline);
-  clutter_timeline_start (priv->timeline);
+  if (priv->transition_duration)
+    clutter_timeline_start (priv->timeline);
+  else
+    create_new_material (image, 1.0);
 
   /* the image has changed size, so update the preferred width/height */
   clutter_actor_queue_relayout (CLUTTER_ACTOR (image));
@@ -1057,11 +1105,11 @@ mx_image_set_from_data_internal (MxImage          *image,
       blank_area = g_new0 (gint, MAX (width, height) + 2);
       cogl_texture_set_region (priv->texture, 0, 0, 0, 0,
                                width, 1, width, 1,
-                               COGL_PIXEL_FORMAT_RGBA_8888, width * 4,
+                               COGL_PIXEL_FORMAT_RGBA_8888, (width + 2) * 4,
                                (const guint8 *)blank_area);
       cogl_texture_set_region (priv->texture, 0, 0, 0, height + 1,
                                width + 2, 1, width + 2, 1,
-                               COGL_PIXEL_FORMAT_RGBA_8888, width * 4,
+                               COGL_PIXEL_FORMAT_RGBA_8888, (width + 2) * 4,
                                (const guint8 *)blank_area);
       cogl_texture_set_region (priv->texture, 0, 0, 0, 0,
                                1, height + 2, 1, height + 2,
@@ -1400,6 +1448,7 @@ mx_image_pixbuf_new (const gchar  *filename,
         {
           if (error)
             g_propagate_error (error, err);
+          gdk_pixbuf_loader_close (loader, NULL);
           g_object_unref (loader);
           return NULL;
         }
@@ -1409,6 +1458,7 @@ mx_image_pixbuf_new (const gchar  *filename,
 
   if (!buffer)
     {
+      gdk_pixbuf_loader_close (loader, NULL);
       g_object_unref (loader);
       return NULL;
     }
@@ -1417,6 +1467,7 @@ mx_image_pixbuf_new (const gchar  *filename,
     {
       if (error)
         g_propagate_error (error, err);
+      gdk_pixbuf_loader_close (loader, NULL);
       g_object_unref (loader);
       return NULL;
     }
@@ -1527,7 +1578,12 @@ mx_image_set_async (MxImage         *image,
   if (!mx_image_threads)
     {
       mx_image_threads = g_thread_pool_new (mx_image_async_cb, NULL,
+#ifdef _SC_NPROCESSORS_ONLN
                                             sysconf (_SC_NPROCESSORS_ONLN),
+#else
+                                            /* FIXME: add more OSs */
+                                            1,
+#endif
                                             FALSE, &err);
       if (!mx_image_threads)
         {
@@ -2162,8 +2218,8 @@ mx_image_set_transition_duration (MxImage *image, guint duration)
     {
       image->priv->transition_duration = duration;
 
-      clutter_timeline_set_duration (CLUTTER_TIMELINE (image->priv->timeline),
-                                     duration);
+      if (duration != 0)
+        clutter_timeline_set_duration (image->priv->timeline, duration);
 
       g_object_notify (G_OBJECT (image), "transition-duration");
     }
